<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://www.dhtmlchess.com
 * @since             1.0.0
 * @package           Dhtml_chess
 *
 * @wordpress-plugin
 * Plugin Name:       DHTML Chess
 * Plugin URI:        http://wordpresschess.com
 * Description:       Display Chess Games and Chess tactic puzzles on your website.
 * Version:           1.0.54
 * Author:            dhtmlchess.com
 * Author URI:        http://www.dhtmlchess.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       dhtml_chess
 * Domain Path:       /languages
 */

define("DHTML_CHESS_VERSION", "1.0.54");
define("DHTML_CHESS_PRO", true);

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}


add_option("dhtml_chess_db_version", 1);
add_option("dhtml_chess_db_installed_2", 0);

require 'plugin-update-checker/plugin-update-checker.php';
$MyUpdateChecker = PucFactory::buildUpdateChecker(
    'http://dhtmlchess.com/api/wordpress_dhtml_chess.json',
    __FILE__,
    'dhtml_chess'
);

// Update this when database changes has been done
$dhtml_chess_db_version = 3;

require_once plugin_dir_path(__FILE__) . 'api/autoload.php';

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-dhtml_chess-activator.php
 */
function activate_dhtml_chess()
{
    require_once plugin_dir_path(__FILE__) . 'includes/class-dhtml_chess-activator.php';
    require_once plugin_dir_path(__FILE__) . 'includes/class-dhtml_chess-upgrade.php';
    Dhtml_chess_Activator::activate();
}

function uninstall_dhtml_chess()
{

    $options = get_option("dhtml_chess");
    $remove_on_uninstall = isset($options["remove_on_uninstall"]) ? $options["remove_on_uninstall"] : '0';

    if ($remove_on_uninstall == '1') {
        require_once plugin_dir_path(__FILE__) . 'includes/class-dhtml_chess-uninstall.php';
        # Dhtml_chess_Uninstall::uninstall();
    }
}

function dhtml_chess_update_db_check()
{
    require_once plugin_dir_path(__FILE__) . 'includes/class-dhtml_chess-upgrade.php';
    Dhtml_chess_Upgrade::upgrade();
}

register_activation_hook(__FILE__, 'activate_dhtml_chess');
# register_deactivation_hook(__FILE__, 'deactivate_dhtml_chess');
register_uninstall_hook(__FILE__, 'uninstall_dhtml_chess');
add_action('plugins_loaded', 'dhtml_chess_update_db_check');

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path(__FILE__) . 'includes/class-dhtml_chess.php';


add_action('admin_post_dhtml_chess_import_pgn', 'dc_import_pgn');
add_action('wp_ajax_get_standings', 'dc_get_standings');
add_action('wp_ajax_get_standings_html', 'dc_get_standings_html');
add_action('wp_ajax_nopriv_get_standings', 'dc_get_standings');
add_action('wp_ajax_count_games', 'dc_count_games');
add_action('wp_ajax_get_draft', 'dc_get_draft');
add_action('wp_ajax_discard_draft', 'dc_discard_draft');
add_action('wp_ajax_save_draft', 'dc_save_draft');
add_action('wp_ajax_list_drafts', 'dc_list_drafts');
add_action('wp_ajax_random_game', 'dc_random_game');
add_action('wp_ajax_nopriv_random_game', 'dc_random_game');
add_action('wp_ajax_game_by_index', 'dc_game_by_index');
add_action('wp_ajax_nopriv_game_by_index', 'dc_game_by_index');
add_action('wp_ajax_game_by_id', 'dc_game_by_id');
add_action('wp_ajax_nopriv_game_by_id', 'dc_game_by_id');
add_action('wp_ajax_list_pgns', 'dc_list_pgns');
add_action('wp_ajax_nopriv_list_pgns', 'dc_list_pgns');
add_action('wp_ajax_list_of_games', 'dc_list_of_games');
add_action('wp_ajax_nopriv_list_of_games', 'dc_list_of_games');
add_action('wp_ajax_nopriv_download_pgn', 'dc_download_pgn');
add_action('wp_ajax_delete_game', 'dc_delete_game');
add_action('wp_ajax_save_game', 'dc_save_game');
add_action('wp_ajax_publish_game', 'dc_publish_game');
add_action('wp_ajax_archive_pgn', 'dc_archive_pgn');
add_action('wp_ajax_new_pgn', 'dc_new_pgn');
add_action('wp_ajax_list_archived', 'dc_list_archived');
add_action('wp_ajax_restore_pgn', 'dc_restore_pgn');
add_action('wp_ajax_delete_pgn', 'dc_delete_pgn');
add_action('wp_ajax_rename_pgn', 'dc_rename_pgn');
add_action('wp_ajax_import_pgn_string', 'dc_import_pgn_string');


$start_time = microtime(true);


function canEditDhtmlChess()
{
    return true;
}


function dc_import_pgn()
{

    /*
     * {"pgn":{"name":"art_attack.pgn","type":"application\/octet-stream","tmp_name":"\/private\/var\/tmp\/phpGyfato","error":0,"size":60118}}
     */
    if (canEditDhtmlChess() && isset($_FILES['pgn'])) {

        check_admin_referer('dhtml_chess_importpgn', '_nonce_dhtml_chess');

        try {
            $pgn = $_FILES['pgn'];

            if (empty($_FILES['pgn']) || empty($pgn["tmp_name"])) {
                throw new DhtmlChessException("No file");
            }
            $name = empty($_POST['pgn_title']) ? $pgn["name"] : $_POST['pgn_title'];

            $pgnContent = file_get_contents($pgn["tmp_name"]);
            $importer = new DhtmlChessImportPgn();
            $pgn = $importer->importPgnStringToNewDatabase($name, $pgnContent);

            if (!empty($pgn)) {
                if (wp_redirect(admin_url(esc_url_raw('admin.php?page=dhtml_chess_import_pgn&m=' . $pgn->getName() . "&pgn_id=" . $pgn->getId())))) {
                    exit;
                };
            }

        } catch (DhtmlChessException $e) {
            if (wp_redirect(admin_url(esc_url_raw('admin.php?page=dhtml_chess_import_pgn&e=1&msg=' . $e->getMessage())))) {
                exit;
            };

        }

        if (wp_redirect(admin_url(esc_url_raw('admin.php?page=dhtml_chess_import_pgn&e=1')))) {
            exit;
        };
    }
    wp_die();
}


/**
 * @param DhtmlChessException $exception
 */
function outputError($exception)
{
    header("content-type: Application/json");
    $message = "Error: " . $exception->getMessage();
    $data = array("success" => false, "response" => $message);
    echo json_encode($data);
}

function outputContent($jsonString)
{
    header("content-type: Application/json");
    global $start_time;
    $elapsed = microtime(true) - $start_time;

    echo '{"success" : true, "time": ' . $elapsed . ', "response":' . $jsonString . '}';
}

function dc_get_standings_html(){
    if (!isset($_POST['pgn'])) {
        throw new DhtmlChessException("Missing pgn attribute");
    }
    $views = new DhtmlChessViews();
    echo $views->standingsAsHTML($_POST['pgn']);
    wp_die();
}

function dc_get_standings()
{
    if (!isset($_POST['pgn'])) {
        throw new DhtmlChessException("Missing pgn attribute");
    }
    $db = new DhtmlChessDatabase();
    outputContent($db->getStandings($_POST["pgn"]));
    wp_die();
}

function dc_download_pgn()
{
    if (isset($_POST["game"])) {
        try {
            $db = new DhtmlChessDatabase();
            $game = $db->gameById($_POST["game"]);
            if(empty($game)){
                throw new DhtmlChessException("Unable to locate pgn");
            }
            $parser = new JsonToPgnParser();
            $parser->addGame($game);
            outputContent($parser->asPgn());
        } catch (DhtmlChessException $e) {
            outputError($e);
        }
    }
    wp_die();
}


function dc_list_drafts()
{
    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->allDrafts());
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}

function dc_count_games()
{
    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->countGamesInDatabase());
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}

function dc_get_draft()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (!isset($_POST['draft_id'])) {
            throw new DhtmlChessException("Missing draft_id");
        }
        $db = new DhtmlChessDatabase();
        outputContent($db->getDraft($_POST['draft_id']));
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}

function dc_list_archived()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        $db = new DhtmlChessDatabase();
        outputContent($db->listOfArchivedPgns());
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}

function dc_restore_pgn()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (empty($_POST['pgn'])) {
            throw new DhtmlChessException("PGN not set");
        }

        $pgnId = preg_replace("/[^0-9]/si", "", $_POST["pgn"]);
        if (empty($pgnId)) {
            throw new DhtmlChessException("Invalid pgn id");
        }

        $db = new DhtmlChessDatabase();
        outputContent($db->restoreArchived($pgnId));
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}

function dc_import_pgn_string()
{
    if (isset($_POST['pgn_id']) && isset($_POST['pgn'])) {
        try {
            $db = new DhtmlChessDatabase();
            $response = $db->appendPgnString($_POST['pgn_id'], $_POST['pgn']);
            $countImported = $response[0];
            $total = $response[1];
            if ($countImported == 0) {
                throw new DhtmlChessException("No games imported");
            }
            $array = array('imported' => $countImported, 'total' => $total);
            $json = json_encode($array);
            outputContent($json);
        } catch (DhtmlChessException $e) {
            outputError($e);
        }
    }

    wp_die();
}


function dc_rename_pgn()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (empty($_POST['pgn'])) {
            throw new DhtmlChessException("PGN not set");
        }
        if (empty($_POST['name'])) {
            throw new DhtmlChessException("PGN not set");
        }

        $newName = trim($_POST['name']);

        $pgnId = preg_replace("/[^0-9]/si", "", $_POST["pgn"]);
        if (empty($pgnId)) {
            throw new DhtmlChessException("Invalid pgn id");
        }

        $db = new DhtmlChessDatabase();
        $pgn = $db->rename($pgnId, $newName);
        outputContent($pgn->asJSON());

    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}

function dc_delete_pgn()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (empty($_POST['pgn'])) {
            throw new DhtmlChessException("PGN not set");
        }

        $pgnId = preg_replace("/[^0-9]/si", "", $_POST["pgn"]);
        if (empty($pgnId)) {
            throw new DhtmlChessException("Invalid pgn id");
        }

        $db = new DhtmlChessDatabase();
        outputContent($db->deletePgn($pgnId));
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}


function dc_new_pgn()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (!isset($_POST['pgn_name'])) {
            throw new DhtmlChessException("Missing pgn name");
        }
        $db = new DhtmlChessDatabase();
        $pgn = $db->createDatabase($_POST['pgn_name']);

        outputContent($pgn->getId());
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}

function dc_archive_pgn()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (!isset($_POST['pgn'])) {
            throw new DhtmlChessException("Missing pgn");
        }

        $db = new DhtmlChessDatabase();
        $pgn = preg_replace("/[^0-9]/si", "", $_POST['pgn']);
        if (empty($pgn)) {
            throw new DhtmlChessException("Missing pgn");
        }
        $success = $db->archivePgn($pgn);
        if ($success) {
            outputContent($success);
        } else {
            throw new DhtmlChessException("Unable to archive - no rows updated");
        }

    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}

function dc_discard_draft()
{

    try {
        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        $draftId = !empty($_POST['draft_id']) ? $_POST['draft_id'] : '';

        $draftId = preg_replace("/[^0-9]/si", "", $draftId);

        if (empty($draftId)) {
            throw new DhtmlChessException("Unable to delete - draft_id missing");
        }

        $db = new DhtmlChessDatabase();
        $countDeleted = $db->deleteDraft($draftId);

        if ($countDeleted > 0) {
            outputContent($countDeleted);
        } else {
            throw new DhtmlChessException("Could not find draft_id");
        }

    } catch (DhtmlChessException $e) {
        outputError($e);
    }

    wp_die();
}

function dc_publish_game()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        if (!isset($_POST['pgn'])) {
            throw new DhtmlChessException("Missing destination pgn");
        }
        if (!isset($_POST['game'])) {
            throw new DhtmlChessException("Missing game");
        }

        $game = stripslashes($_POST['game']);
        $game = json_decode($game, true);
        $db = new DhtmlChessDatabase();
        $id = $db->publishDraft(json_encode($game), $_POST['pgn']);


        outputContent($id);
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}


function dc_save_game()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        $db = new DhtmlChessDatabase();

        if (!isset($_POST['game'])) {
            throw new DhtmlChessException("Game data missing");

        }


        $game = $_POST['game'];
        $game = stripslashes($game);
        $game = json_decode($game, true);

        if (!empty($_POST['pgn'])) {
            $pgn = $_POST['pgn'];
        } else {
            $pgn = isset($game['pgn']) ? $game['pgn'] : null;
        }

        if (empty($pgn)) {
            throw new DhtmlChessException("Name of pgn for game to saved missing");
        }

        $db->updateGame($pgn, json_encode($game));


        outputContent($game['id']);

    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}


function dc_save_draft()
{
    try {

        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access Denied");
        }

        $game = stripslashes($_POST['game']);
        $game = json_decode($game, true);
        $db = new DhtmlChessDatabase();
        $draftId = $db->saveDraft(json_encode($game));
        $response = array("draft_id" => $draftId);

        outputContent(json_encode($response));
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();

}


function dc_random_game()
{

    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->randomGame($_POST["pgn"]));
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}

function dc_list_of_games()
{
    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->listOfGames($_POST["pgn"]));
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}

function dc_game_by_index()
{


    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->gameByIndex($_POST["pgn"], $_POST["index"]));

    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}

function dc_game_by_id()
{
    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->gameById($_POST["id"]));

    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}

function dc_list_pgns()
{
    try {
        $db = new DhtmlChessDatabase();
        outputContent($db->listOfPgns());

    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}


function dc_delete_game()
{
    try {
        if (!canEditDhtmlChess()) {
            throw new DhtmlChessException("Access denied");
        } else {
            if (!isset($_POST['pgn'])) {
                throw new DhtmlChessException("Missing pgn attribute");
            }
            if (!isset($_POST['id'])) {
                throw new DhtmlChessException("Missing id");
            }

            $db = new DhtmlChessDatabase();
            $count = $db->deleteGame($_POST['pgn'], $_POST["id"]);
            return json_encode(array("success" => $count > 0));
        }
    } catch (DhtmlChessException $e) {
        outputError($e);
    }
    wp_die();
}


/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_dhtml_chess()
{

    $plugin = new Dhtml_chess();
    $plugin->run();

}

run_dhtml_chess();
